@@ -5,23 +5,24 @@ from __future__ import unicode_literals
 from __future__ import with_statement
 
 import abc
-import datetime
-import pprint
 import sys
+import pprint
+import datetime
 
 from python_utils import converters
 
+import six
+
 from . import base
-from . import six
 from . import utils
 
-MAX_DATE = datetime.date(year=datetime.MAXYEAR, month=12, day=31)
-MAX_TIME = datetime.time(23, 59, 59)
-MAX_DATETIME = datetime.datetime.combine(MAX_DATE, MAX_TIME)
+MAX_DATE = datetime.date.max
+MAX_TIME = datetime.time.max
+MAX_DATETIME = datetime.datetime.max
 
 
 def string_or_lambda(input_):
-    if isinstance(input_, six.basestring):
+    if isinstance(input_, six.string_types):
         def render_input(progress, data, width):
             return input_ % data
 
@@ -39,7 +40,7 @@ def create_marker(marker):
         else:
             return marker
 
-    if isinstance(marker, six.basestring):
+    if isinstance(marker, six.string_types):
         marker = converters.to_unicode(marker)
         assert len(marker) == 1, 'Markers are required to be 1 char'
         return _marker
@@ -198,7 +199,7 @@ class FormatLabel(FormatWidgetMixin, WidthWidgetMixin):
                     data[name] = data[key]
                 else:
                     data[name] = transform(data[key])
-            except:  # pragma: no cover
+            except (KeyError, ValueError, IndexError):  # pragma: no cover
                 pass
 
         return FormatWidgetMixin.__call__(self, progress, data, **kwargs)
@@ -215,8 +216,39 @@ class Timer(FormatLabel, TimeSensitiveWidgetBase):
     format_time = staticmethod(utils.format_time)
 
 
-class SamplesMixin(object):
-    def __init__(self, samples=10, key_prefix=None, **kwargs):
+class SamplesMixin(TimeSensitiveWidgetBase):
+    '''
+    Mixing for widgets that average multiple measurements
+
+    Note that samples can be either an integer or a timedelta to indicate a
+    certain amount of time
+
+    >>> class progress:
+    ...     last_update_time = datetime.datetime.now()
+    ...     value = 1
+    ...     extra = dict()
+
+    >>> samples = SamplesMixin(samples=2)
+    >>> samples(progress, None, True)
+    (None, None)
+    >>> progress.last_update_time += datetime.timedelta(seconds=1)
+    >>> samples(progress, None, True)
+    (datetime.timedelta(0, 1), 0)
+    >>> progress.last_update_time += datetime.timedelta(seconds=1)
+    >>> samples(progress, None, True)
+    (datetime.timedelta(0, 1), 0)
+
+    >>> samples = SamplesMixin(samples=datetime.timedelta(seconds=1))
+    >>> _, value = samples(progress, None)
+    >>> value
+    [1, 1]
+
+    >>> samples(progress, None, True)
+    (datetime.timedelta(0, 1), 0)
+    '''
+
+    def __init__(self, samples=datetime.timedelta(seconds=2), key_prefix=None,
+                 **kwargs):
         self.samples = samples
         self.key_prefix = (self.__class__.__name__ or key_prefix) + '_'
 
@@ -226,7 +258,7 @@ class SamplesMixin(object):
     def get_sample_values(self, progress, data):
         return progress.extra.setdefault(self.key_prefix + 'sample_values', [])
 
-    def __call__(self, progress, data):
+    def __call__(self, progress, data, delta=False):
         sample_times = self.get_sample_times(progress, data)
         sample_values = self.get_sample_values(progress, data)
 
@@ -240,11 +272,25 @@ class SamplesMixin(object):
             sample_times.append(progress.last_update_time)
             sample_values.append(progress.value)
 
-            if len(sample_times) > self.samples:
-                sample_times.pop(0)
-                sample_values.pop(0)
+            if isinstance(self.samples, datetime.timedelta):
+                begin = progress.last_update_time - self.samples
+                while sample_times[2:] and begin > sample_times[0]:
+                    sample_times.pop(0)
+                    sample_values.pop(0)
+            else:
+                if len(sample_times) > self.samples:
+                    sample_times.pop(0)
+                    sample_values.pop(0)
 
-        return sample_times, sample_values
+        if delta:
+            delta_time = sample_times[-1] - sample_times[0]
+            delta_value = sample_values[-1] - sample_values[0]
+            if delta_time:
+                return delta_time, delta_value
+            else:
+                return None, None
+        else:
+            return sample_times, sample_values
 
 
 class ETA(Timer):
@@ -349,15 +395,12 @@ class AdaptiveETA(ETA, SamplesMixin):
         SamplesMixin.__init__(self, **kwargs)
 
     def __call__(self, progress, data):
-        times, values = SamplesMixin.__call__(self, progress, data)
+        elapsed, value = SamplesMixin.__call__(self, progress, data,
+                                               delta=True)
 
-        if len(times) <= 1:
-            # No samples so just return the normal ETA calculation
+        if not elapsed:
             value = None
             elapsed = 0
-        else:
-            value = values[-1] - values[0]
-            elapsed = utils.timedelta_to_seconds(times[-1] - times[0])
 
         return ETA.__call__(self, progress, data, value=value, elapsed=elapsed)
 
@@ -448,15 +491,8 @@ class AdaptiveTransferSpeed(FileTransferSpeed, SamplesMixin):
         SamplesMixin.__init__(self, **kwargs)
 
     def __call__(self, progress, data):
-        times, values = SamplesMixin.__call__(self, progress, data)
-        if len(times) <= 1:
-            # No samples so just return the normal transfer speed calculation
-            value = None
-            elapsed = None
-        else:
-            value = values[-1] - values[0]
-            elapsed = utils.timedelta_to_seconds(times[-1] - times[0])
-
+        elapsed, value = SamplesMixin.__call__(self, progress, data,
+                                               delta=True)
         return FileTransferSpeed.__call__(self, progress, data, value, elapsed)
 
 
